/* recfilte.c: tty record filter */
/* $Id: recfilter.c,v 1.5 2003/11/21 09:44:10 knok Exp $ */

/*
Copyright (C) 2003 NOKUBI Takatsugu <knok@daionet.gr.jp>
This is free software with ABSOLUTELY NO WARRANTY.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include <termios.h>
#include <signal.h>

#include "gnet.h"

void settermraw(void);
void resetterm(void);
void sighandler(int);

static struct termios tbuf;

static GUnixSocket *gs;
static GIOChannel *io;

#define MAXLINE 2048

int
main(int argc, char *argv[])
{
	char	result[MAXLINE];
	char	local_echo[MAXLINE];
	int	tc_mode;
	unsigned char uc;
	GIOError ge;

	if (isatty(0)) {
	  settermraw();
	  setbuf(stdout, NULL);
	}
	gs = gnet_unix_socket_new("/tmp/backtick");
	if (gs == NULL)
	    return 0;
	io = gnet_unix_socket_get_io_channel(gs);
	/* main loop */
	while (read(0, &uc, 1) == 1) {
	  if (uc >= 30 && uc <= 0x7e) {
	    char tmp[1024];
	    gsize foo;
	    sprintf(tmp, "%c\n", uc);
	    ge = gnet_io_channel_writen(io, tmp, 2, &foo);
	    if (ge != G_IO_ERROR_NONE)
		break;
	  }
	  fputc(uc, stdout);
	}

	resetterm();
        return 0;
}

void settermraw()
{
  struct termios term;
  if (tcgetattr(0, &term) < 0)
    return;
  memcpy(&tbuf, &term, sizeof(tbuf));

  signal(SIGINT, sighandler);
  signal(SIGQUIT, sighandler);
  signal(SIGTERM, sighandler);
  signal(SIGHUP, sighandler);

  term.c_lflag &= ~(ICANON | ISIG | ECHO | IEXTEN);
  term.c_iflag &= ~(INPCK | ICRNL | ISTRIP | IXON | BRKINT);
  term.c_oflag &= ~OPOST;
  term.c_cflag &= ~(CSIZE | PARENB);
  term.c_cflag |= CS8;
  term.c_cc[VMIN] = 1;
  term.c_cc[VTIME] = 0;
  tcsetattr(0, TCSAFLUSH, &term);
}

void resetterm()
{
  if (tcgetattr(0, &tbuf) < 0)
    return;
}

void sighandler(int s)
{
  resetterm();
  exit(0);
}
